/*
  Vocatra, a small and fast vocabulary trainer
  Copyright (C) 2006-2008 Oliver Sauder <os@esite.ch>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _VOCADMIN_H
#define _VOCADMIN_H

/** Vocabulary Administration class.
 *  This class controls the learning process.
 *  It administrates a bit array with the current learning status
 *  and handles the current learning position as well.
 */
class CVocAdmin
{
private:
	/** Helper function.
	 *  This function prints the learning bit array to the standard output,
	 *  if the debug mode is enabled.
	 */
	void printBits();
  
	/** Helper function.
	 *  This function initializes a second learning bit array.
	 *  CVocAdmin will need this array, if the stack mode is enabled.
	 *  @see MarkAsLearnedStack
	 */
	void initArrStack();

public:
	/** Constructor.
	 *  This constructor initializes a bit array with numWords positions.
	 *  and sets the debug mode.
	 *  @param nNumWords size from the learning bit array.
	 *  @param bDebug enable or disable the debug mode.
	 */
	CVocAdmin(const size_t &nNumWords,const bool &bDebug);

	/** Destructor.
	 *  This destructor releases the allocated memory.
	 */
	~CVocAdmin();

	/** Get the next learning position.
	 *  This function returns a unlearned position x.
	 *  If the random mode is disabled, x is equal the current position + 1.
	 *  @param bRandom enable or disable the random mode.
	 *  @result This function returns the index number from the next learning position.
	 */
	size_t GetLearnPos(bool bRandom);

	/** Mark position idx as learned in the bit array stack.
	 *  This function changes the bit idx in the learning bit array stack to 1.
	 *  It will be used, if vocatra is not in the last stack
	 *  and the stack by stack mode is enabled.
	 *  @param idx learned position in the current array stack.
	 *  @see MarksAsLearned()
	 *  @see Finish()
	 */
	void MarkAsLearnedStack(size_t idx);

	/** Mark position idx as learned in the bit array.
	 *  This function changes the bit idx in the learning bit array to 1.
	 *  It will be used, if the stack by stack mode is disabled
	 *  or vocatra is in the last stack.
	 *  @param idx learned position in the current array.
	 *  @see MarkAsLearnedStack()
	 *  @see Finish()
	 */
	void MarkAsLearned(size_t idx);
  
	/** Is learning process finished function.
	 *  This function returns true, if every position in the learning bit array is 1.
	 *  @result This function returns true, if the learning process is finished; otherwise false.
	 */
	bool IsFinished();

private:
	/** Learned bit array with m_nNumArrDim dimensions.
	 *  Every dimension contains 32 positions and every position resp. bit
	 *  represents a word.
	 *  If the bit is set to 1 means that the word is already learned.
	 *  @see m_nNumArrDim
	 *  @see MarkAsLearned()
	 */
	size_t *m_pBitArrLearned;

	/** Learned bit array stack with m_nNumArrDim dimensions.
	 *  @see m_nNumArrDim
	 *  @see m_npBitArrLearned
	 *  @see MarkAsLearnedStack()
	 */
	size_t *m_pBitArrLearnedStack;

	/** number of unlearned words */
	size_t m_nNumUnlearnedWords;

	/** number of unlearned words in the current stack */
	size_t m_nNumUnlearnedWordsStack;

	/** Number of array dimensions in m_pBitArrLearned and m_pBitArrLearnedStack.
	 *  @see m_pBitArrLearned
	 */
	size_t m_nNumArrDim;

	/** number of the last learn position */
	size_t m_nLastLearnPos;

	/** debug mode */
	bool m_bDebug;
};

#endif // _VOCADMIN_H
