/*
  Vocatra, a small and fast vocabulary trainer
  Copyright (C) 2006-2008 Oliver Sauder <os@esite.ch>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <cstdlib>

#include "vocatra.H"
#include "vocatrainfo.H"
#include "vocadmin.H"

using namespace std;

#define SZ_BIT (sizeof(size_t)*8)

CVocAdmin::CVocAdmin(const size_t &nNumWords,const bool &bDebug)
{
	m_nNumUnlearnedWords=nNumWords;
	m_nNumArrDim=nNumWords/SZ_BIT+1;
	m_pBitArrLearned=new size_t[m_nNumArrDim];
	m_pBitArrLearnedStack=new size_t[m_nNumArrDim];
	m_bDebug=bDebug;

	// init array as unlearned
	for(size_t i=0;i<m_nNumArrDim;++i)
    {
		m_pBitArrLearned[i]=0;
    }

	// init not available words as learned
	for(size_t i=nNumWords-(m_nNumArrDim-1)*SZ_BIT;i<SZ_BIT;++i)
    {
		m_pBitArrLearned[m_nNumArrDim-1]|=(1<<i);
    }

	initArrStack();
}

CVocAdmin::~CVocAdmin()
{
	delete[] m_pBitArrLearned;
	delete[] m_pBitArrLearnedStack;
}

void CVocAdmin::printBits()
{
	if(m_bDebug)
    {
		cout << CVocatraInfo::GetName() << ": ";
		cout << "Debug: BitArrayLearned: ";
		for(size_t i=m_nNumArrDim;i>0;--i)
		{
			for(size_t j=SZ_BIT;j>0;--j)
			{
				cout << ((m_pBitArrLearned[i-1]&(1<<j-1))>0);
			}
	  
			if(i>1)
				cout << " ";
		}
		cout << endl;
    }
}

void CVocAdmin::initArrStack()
{
	for(size_t i=0;i<m_nNumArrDim;++i)
    {
		m_pBitArrLearnedStack[i]=m_pBitArrLearned[i];
    }

	m_nNumUnlearnedWordsStack=m_nNumUnlearnedWords;
	m_nLastLearnPos=m_nNumArrDim*SZ_BIT-1;
}

size_t CVocAdmin::GetLearnPos(bool bRandom)
{
	size_t num;
	if(bRandom) {
		num=random()%m_nNumUnlearnedWordsStack;
    }
	else {
		num=0;
    }
  
	++num;
	size_t i=m_nLastLearnPos;
	do {
		if((i+1)>=m_nNumArrDim*SZ_BIT) {
			i=0;
		}
		else {
			++i;
		}
    
		int nNumArrDim=i/SZ_BIT;
		int nNumBit=i-(nNumArrDim*SZ_BIT);
		if((m_pBitArrLearnedStack[nNumArrDim]&(1<<nNumBit))==0) {
			--num;
	
			if(num==0) {
				m_nLastLearnPos=i;
				return i;
			}
		}
	} while(i!=m_nLastLearnPos);

	assert(false);  // cannot find a unlearned word
	m_nLastLearnPos=0;
	return 0;
}

void CVocAdmin::MarkAsLearnedStack(size_t idx)
{
	int nNumArrDim=idx/SZ_BIT;
	int nNumBit=idx-(nNumArrDim*SZ_BIT);
	m_pBitArrLearnedStack[nNumArrDim]|=(1<<nNumBit);
      
	--m_nNumUnlearnedWordsStack;

	if(m_nNumUnlearnedWordsStack==0) {
		initArrStack();
    }
}

void CVocAdmin::MarkAsLearned(size_t idx)
{
	int nNumArrDim=idx/SZ_BIT;
	int nNumBit=idx-(nNumArrDim*SZ_BIT);
	m_pBitArrLearned[nNumArrDim]|=(1<<nNumBit);
      
	if(m_nNumUnlearnedWords>0) {
		--m_nNumUnlearnedWords;
	}
}

bool CVocAdmin::IsFinished()
{
	printBits();
	return !m_nNumUnlearnedWords;
}
