/*
  Vocatra, a small and fast vocabulary trainer
  Copyright (C) 2006-2008 Oliver Sauder <os@esite.ch>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _VOCFILE_H
#define _VOCFILE_H

#include "vocstruct.H"

/** Vocabulary File class.
 *  This class reads a vocabulary file and appends the words to a vector.
 */
class CVocFile
{
private:
	/** Helper function.
	 *  This function trims spaces and tabulators from a string.
	 *  @param str This string will be trimmed.
	 */
	void trim(std::string &str);

	/** Helper function.
	 *  This function trims a line and checks if this line is in the correct format.
	 *  @param line This string will be checked.
	 *  @result
	 *    Returns the position from the equal sign. If the return value is 0 the line
	 *    is commented out or empty. If the return value is string::npos the line is
	 *    incorrect.
	 *  @see trim
	 */
	size_t checkLine(std::string &line);

	/** Helper function.
	 *  This function separates a string into words. The separation sign is a comma.
	 *  @param str This string will be separated.
	 *  @param words Words will be assigned to this vector.
	 */
	void seperateWords(std::string str,std::vector<std::string> &words);

    /** Helper function.
	 *  This function adds a word to a vocabulary vector.
	 *  If the synonym mode is enabled, it will also split one vocabulary word 
	 *  into n-lang-synonym vocabulary items.
	 *  @param word This word will be added to a vocabulary vector.
	 *  @param vocabulary Word will be appended to this vocabulary vector.
	 *  @param bSynonym synonym mode
	 *  @param enLang inquiry language
	 */
	void addWordToVocabulary(const SWord &word,std::vector<SWord> &vocabulary,
							 const bool &bSynonym,const enVocLang &enLang);

public:
	/** Constructor.
	 *  This constructor initializes the vocabulary filename.
	 *  @param filename vocabulary filename
	 */
	CVocFile(char* filename);

	/** Append Vocabulary function.
	 *  This function reads a vocabulary file and appends the words to a vector.
	 *  A error message will be written to the stderr if a line is incorrect.
	 *  If the synonym mode is enabled, it will also split one vocabulary word 
	 *  resp. one line into n-lang-synonym items.
	 *  @param vocabulary Words will be appended to this vector.
	 *  @param bSynonym synonym mode
	 *  @param enLang inquiry language
	 */
	void AppendVocabulary(std::vector<SWord> &vocabulary,
						  const bool &bSynonym,const enVocLang &enLang);


private:
	/** vocabulary filename */
	std::string m_strFilename;
};

#endif //_VOCFILE_H
