/*
  Vocatra, a small and fast vocabulary trainer
  Copyright (C) 2006-2008 Oliver Sauder <os@esite.ch>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _VOCOPTION_H
#define _VOCOPTION_H

#include "vocstruct.H"

/** Vocabulary Options class.
 *  This class parses the command line and makes the options available for other classes.
 */
class CVocOptions
{
public:
	/** Default constructor.
	 *  This constructor initializes the member variables with default values.
	 */
	CVocOptions();
  
public:
	/** Parser function.
	 *  This function parses the command line and fills the member variables.
	 *  @result The operation was unsuccessful if the return value is unequal -1.
	 */
	int Parse(int argc,char *argv[]);

	/** Debug mode function.
	 *  This function returns the current debug mode setting.
	 *  @result This function returns true if the debug mode is enabled; otherwise false.
	 */
	bool DebugMode() const {
		return m_bDebug;
	};

	/** Stack mode function.
	 *  This function returns the current stack mode setting.
	 *  @result This function returns true if the stack mode is enabled; otherwise false.
	 */
	bool StackMode() const {
		return m_bStack;
	};

	/** Case mode function.
	 *  This function returns the current case mode setting.
	 *  @result This function returns true if the mode is case-sensitive and false if case-insensitive.
	 */
	bool CaseMode() const {
		return m_bCaseMode;
	};

	/** Synonym mode function.
	 * This function returns the current synonym mode setting.
	 * @result This function returns true if the synonym mode is enabled; otherwise false.
	 */
	bool SynonymMode() const {
		return m_bSynonym;
	}

	/** Beep mode function
	 *  This function returns the current beep mode settings.
	 *  @result This function returns true if the beep mode is enabled; otherwise false.
	 */
	bool BeepMode() const {
		return m_bBeep;
	}

	/** Get inquiry mode function.
	 *  This function returns the current inquiry mode setting.
	 *  @result This function returns vmOrder if the inquiry mode is order,
	            vmRandom if random and vmAlphabetical if alphabetical.
	 */
	enVocMode GetMode() const {
		return m_enMode;
	};

	/** Get language mode function.
	 *  This function returns the current inquiry language setting.
	 *  @result This function returns vlLeft if the inquiry language is left,
	 *          vlRight if right.
	 */
	enVocLang GetLang() const {
		return m_enLang;
	};

	/** Get number of passes function.
	 *  @result This function returns the number of passes setting.
	 */
	int GetNumOfPasses() const {
		return m_nNumPasses;
	};

	/** Get name from vocabulary files function.
	 *  This function returns a array with vocabulary files.
	 *  @result This functions returns a null-terminated character array.
	 *  @see GetNumOfVocFiles()
	 */
	char** GetVocFiles() const {
		return m_pVocFiles;
	};
  
	/** Get number of vocabulary files function.
	 *  @result This function returns number of vocabulary files.
	 *  @see GetVocFiles()
	 */
	int GetNumOfVocFiles() const {
		return m_nNumFiles;
	};

private:
	/** debug mode boolean */
	bool m_bDebug;
	
	/** stack mode boolean */
	bool m_bStack;

	/** case-sensitive boolean */
	bool m_bCaseMode;

	/** synonym boolean */
	bool m_bSynonym;
	
	/** beep boolean */
	bool m_bBeep;

	/** mode enum.
	 *  This value will be vmOrder if the inquiry mode is order,
	 *  vmRandom if random and vmAlphabetical if alphabetical.
	 */
	enVocMode m_enMode;

    /** language enum.
	 *  This value will be vlLeft if the inquiry language is left;
	 *  vlRight if right.
	 */
	enVocLang m_enLang;

	/** number of passes */
	int m_nNumPasses;
	
	/** null-terminated voc-file list */
	char **m_pVocFiles;

	/** number of vocabulary files */
	int m_nNumFiles;
};

#endif //_VOCOPTION_H
