/*
  Vocatra, a small and fast vocabulary trainer
  Copyright (C) 2006-2008 Oliver Sauder <os@esite.ch>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _VOCTRAINER_H
#define _VOCTRAINER_H

#include "vocstruct.H"
#include "vocprinter.H"

/** Vocabulary Trainer class.
 *  This class provides the training process. It starts the learning process
 *  and checks the user-input.
 */
class CVocTrainer
{
private:
	/** Helper function.
	 *  This function compares two strings case-sensitively or case-insensitively.
	 *  @param str1 string one
	 *  @param str2 string two
	 *  @param casemode case-sensitive setting
	 *  @result This function returns true, if the strings are equal; otherwise false.
	 */
	bool compareStrings(std::string str1,std::string str2,const bool &casemode);

	/** Helper function.
	 *  This function checks if the word is in the vector words.
	 *  @param word word to check
	 *  @param words vector with words
	 *  @param casemode case-sensitive setting
	 *  @result This function returns true, if the word is in the vector; otherwise false.
	 *  @see compareStrings()
	 */
	bool checkWord(const std::string &word,const std::vector<std::string> &words,
				   const bool &casemode);

	/** Static helper function.
	 *  This function compares two words from the left language.
	 *  @param word1 first word
	 *  @param word2 second word
	 *  @result
	 *    This function returns false, if word1.lang1[0] is less than
	 *    word2.lang1[0]; otherwise true.
	 *  @see sortVocabulary()
	 */
	static bool compareLeftLanguage(const SWord &word1,const SWord &word2);

	/** Static helper function.
	 *	This function compare two words from the right language.
	 *  @param word1 first word
	 *  @param word2 second word
	 *  @result
	 *    This function returns false, if word1.lang2[0] is less than
	 *    word2.lang2[0]; otherwise true.
	 *  @see sortVocabulary()
	 */
	static bool compareRightLanguage(const SWord &word1,const SWord &word2);

	/** Helper function
	 *  This function sorts the vocabulary alphabetically.
	 *  @param
	 *    enLang If enLang is vlRight the right language will be sorted;
	 *    otherwise the left one.
	 */
	void sortVocabulary(const enVocLang &enLang);
  
public:
	/** Constructor.
	 *  @param pvpPrinter Pointer to CVocPrinter class which will be used to write strings
	 */
	CVocTrainer(CVocPrinter *pvpPrinter);

	/** Default destructor.
	 */
	~CVocTrainer();

	/** Initialization function.
	 *  This function initializes the m_vocabulary vector with the help of CVocFile.
	 *  @param files null-terminated file string array
	 *  @param nNumFiles number of files
	 *  @param bSynonym synonym mode
	 *  @param enLang inquiry language
	 *  @result This function returns -1 if successful; otherwise a exit code.
	 *  @see CVocFile
	 *  @see m_vocabulary
	 */
	int Init(char** files,const int &nNumFiles,const bool &bSynonym,const enVocLang &enLang);

	/** Learning function.
	 *  This function starts the learning process and queries the user with the help of CVocAdmin.
	 *  @param enMode inquiry mode
	 *  @param enLang inquiry language
	 *  @param nNumPasses number of passes
	 *  @param bDebugMode debug mode setting
	 *  @param bStackMode stack mode setting
	 *  @param bCaseMode case-sensitive mode setting
	 *  @param bBeep enable beep sound
	 *  @see CVocAdmin
	 */
	void Learn(const enVocMode &enMode,const enVocLang &enLang,
			   const int &nNumPasses,const bool &bDebugMode,
			   const bool &bStackMode,const bool &bCaseMode,
			   const bool &bBeep);
  
private:
	/** vector with the hole vocabulary */
	std::vector<SWord> m_vocabulary;

	/** pointer to printer class */
	CVocPrinter *m_pvpPrinter;
};

#endif //_VOCTRAINER_H
