/*
  Vocatra, a small and fast vocabulary trainer
  Copyright (C) 2006-2008 Oliver Sauder <os@esite.ch>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "vocatra.H"
#include "vocfile.H"
#include "vocatrainfo.H"
#include "vocadmin.H"
#include "voctrainer.H"

#include <string>
#include <iostream>
#include <algorithm>

using namespace std;

bool CVocTrainer::compareStrings(std::string str1,std::string str2,
								 const bool &casemode)
{
	if(!casemode)
    { // case-insensitive
		transform(str1.begin(),str1.end(),str1.begin(),(int(*)(int))tolower);
		transform(str2.begin(),str2.end(),str2.begin(),(int(*)(int))tolower);
    }
	return str1==str2;
}

bool CVocTrainer::checkWord(const std::string &word,const std::vector<std::string> &words,
							const bool &casemode)
{
	for(size_t i=0;i<words.size();++i)
    {
		if(compareStrings(word,words[i],casemode))
			return true;
    }
	return false;
}

bool CVocTrainer::compareLeftLanguage(const SWord &word1,const SWord &word2)
{
	if(word1.lang1.empty()||word2.lang1.empty()) {
		assert(false);			// should not happen
		return false;
	}

	return word1.lang1[0]<word2.lang1[0];
}

bool CVocTrainer::compareRightLanguage(const SWord &word1,const SWord &word2)
{
	if(word1.lang2.empty()||word2.lang2.empty()) {
		assert(false);			// should not happen
		return false;
	}

	return word1.lang2[0]<word2.lang2[0];
}

void CVocTrainer::sortVocabulary(const enVocLang &enLang)
{
	if(enLang==vlLeft) {
		stable_sort(m_vocabulary.begin(),m_vocabulary.end(),&compareLeftLanguage);
	}
	else if(enLang==vlRight) {
		stable_sort(m_vocabulary.begin(),m_vocabulary.end(),&compareRightLanguage);
	}
	else {
		assert(false);  // should not happen
	}
}

CVocTrainer::CVocTrainer(CVocPrinter *pvpPrinter)
	: m_pvpPrinter(pvpPrinter)
{
}

CVocTrainer::~CVocTrainer()
{
	delete m_pvpPrinter;
}

int CVocTrainer::Init(char** files,const int &nNumFiles,const bool &bSynonym,const enVocLang &enLang)
{
	for(int i=0;i<nNumFiles;++i)
    {
		CVocFile file(files[i]);
		file.AppendVocabulary(m_vocabulary,bSynonym,enLang);
    }

	if(m_vocabulary.empty())
    {
		cerr << CVocatraInfo::GetName() << ": " << _("no vocabulary words loaded") << endl;
		return 1;
    }

	return -1;
}

void CVocTrainer::Learn(const enVocMode &enMode,const enVocLang &enLang,
						const int &nNumPasses,const bool &bDebugMode,
						const bool &bStackMode,const bool &bCaseMode,
						const bool &bBeep)
{
	// start timer
	time_t startTime;
	time(&startTime);
	m_pvpPrinter->PrintStartString();
	
	// if alphabetical mode, sort vocabulary vector
	if(enMode==vmAlphabetical) {
		sortVocabulary(enLang);
	}

	CVocAdmin admin(m_vocabulary.size(),bDebugMode);
	bool bModeRandom=enMode==vmRandom;
	size_t idx,wrong=0,inquiries=0;
	while(!admin.IsFinished())
    {
		idx=admin.GetLearnPos(bModeRandom);

		vector<string> *outLang,*inLang;
		switch(enLang)
		{
		default:
		case vlLeft:
			outLang=&m_vocabulary[idx].lang1;
			inLang=&m_vocabulary[idx].lang2;
			break;
		case vlRight:
			outLang=&m_vocabulary[idx].lang2;
			inLang=&m_vocabulary[idx].lang1;
			break;
		}
      
		// output
		m_pvpPrinter->PrintWords(*outLang);
		
		// increase number of in queries
		++inquiries;
  
		// input word
		string strInput;
		while(true)
		{
			char c;
			cin.get(c);
      
			if( c != '\n' )
				strInput += c;
			else
				break;
		}

		// check input
		if(checkWord(strInput,*inLang,bCaseMode)) {
			m_pvpPrinter->PrintCorrectString();
			++m_vocabulary[idx].numPasses;
			
			// if stack mode is enabled, mark word as learned in the
			// current stack
			if(bStackMode) {
				admin.MarkAsLearnedStack(idx);
			}
		}
        else {
			m_pvpPrinter->PrintWrongString(*inLang,bBeep);

			// increase number of wrong answers
			++wrong;
		}

		if(m_vocabulary[idx].numPasses==nNumPasses) {
			// if stackmode is disabled, mark word as learned in the
			// stack (only one stack available)
			if(!bStackMode) {
				admin.MarkAsLearnedStack(idx);
			}

			admin.MarkAsLearned(idx);
		}
    }

	// stop timer
	time_t endTime;
	time(&endTime);

	m_pvpPrinter->PrintEndString(m_vocabulary.size(),inquiries,wrong,size_t(endTime-startTime));
}
